mod Mxml

rem 
  \para \bf  [CL] CL-TeX.  \end 

appldisp/1 Tex1_lst
  Fenced(Op('[',0),Arg(0),Op(']',0))

appldisp/1 Tex1_lx
  Std('[X]',0)

appldisp/2 Tex2_type
  Infix(Arg(0),0,1,Op('::',0),Arg(1))

appldisp/1 Tex1_paren
  Fenced(Op('(',0),Arg(0),Op(')',0))

rem 
  \para \bf  Attributes of XML elements.  \end Type declaration(s): 
  \eq* 
    Attr = Tex1_paren(Str,Str)
  \end
  \eq* 
    Lattr = Tex1_lst(Attr)
  \end

pred Attr 
  Attr(n,v) <- Str(n) & Str(v)

pred Lattr 
  Lattr(0)
  Lattr(a,as) <- Attr(a) & Lattr(as)

rem 
  \para \bf  Constructors of XML elements.  \end Type signature(s): 
  \eq* 
    Tex2_type(Xe,(Tex1_paren(Str,Tex1_lst(Attr),Tex1_lst(X)) -> X))
  \end
  \eq* 
    Tex2_type(Xs,(Str -> X))
  \end

fun/3 Xe 
  Xe(name,attrs,childxnodes) = 0,name,attrs,childxnodes

fun Xs 
  Xs(text) = 1,text

rem 
  \para \bf  Type checking.  \end Type declaration(s): 
  \eq* 
    Lx = Tex1_lst(X)
  \end
  \para Use both formats for checking type errors. 

pred Lx 
  Lx(0)
  Lx(Xe(n,as,cs),xs) <- Str(n) & Lattr(as) & Lx(cs) & Lx(xs)
  Lx(Xs(t),xs) <- Str(t) & Lx(xs)

pred X 
  X Xe(n,as,cs) <- Str(n) & Lattr(as) & Lx(cs)
  X Xs(t) <- Str(t)

rem 
  \para \bf  XML documents.  \end Type declaration(s): 
  \eq* 
    Xml = Tex1_lst(X)
  \end
  \para We require that \ft L(xs) = 1 \end for every correct value 
  \ft Tex2_type(xs,Xml) \end. 

pred Xml 
  Xml(0)
  Xml(Xe(n,as,cs),xs) <- Str(n) & Lattr(as) & Xml(cs) & Xml(xs)
  Xml(Xs(t),xs) <- Str(t) & Xml(xs)

rem 
  \para \bf  XHTML document skeleton.  \end \header* fun/2 Xhtml  \end Type 
  signature(s): 
  \eq* 
    Tex2_type(Xhtml,(Tex1_paren(Str,Lx) -> Xml))
  \end
  \para The application \ft Xhtml(title,bodycs) \end creates the skeleton of a 
  XHTML document: 
  \verbatim 
    <html xmlns="http://www.w3.org/1999/xhtml">
      <head>
        <title>***</title>
      </head>
      <body>
      ###
      </body>
    </html>
  \end
  \para Note that \ft L Xhtml(title,bodycs) = 1 \end. 

fun/2 Xhtml 
  Xhtml(title,bodycs) = 
  Xe('html',(('xmlns','http://www.w3.org/1999/xhtml'),0),h,b,0),0 <- 
    Xe('head',0,Xe('title',0,Xs(title),0),0) = h & Xe('body',0,bodycs) = b

rem 
  \para \bf  Displaying numbers in decimal notation.  \end Type signature(s): 
  \eq* 
    Tex2_type(N2str,(N -> Str))
  \end
  \eq* 
    Tex2_type(N2x,(N -> X))
  \end
  \eq* 
    Tex2_type(N2lx,(N -> Lx))
  \end

fun/2 N2str1 
  N2str1(10*q+r,s) = r+"0",s <- 0 <= r & r <= 9 & q = 0
  N2str1(10*q+r,s) = N2str1(q,r+"0",s) <- 0 <= r & r <= 9 & q != 0

fun N2str 
  N2str(n) = N2str1(n,0)

fun N2x 
  N2x(n) = Xs N2str(n)

fun N2lx 
  N2lx(n) = N2x(n),0

